@@ -277,9 +277,13 @@ def commit_file(pid, cid):
         commit = md.Commit.get(id=cid)
         commit_file = md.CommitFile.get(commit=commit, file_path=fpath)
     except:
-        abort(404)
-    diff = commit.project.git_control.get_file_diff(
-        commit.sha1, commit_file.file_path)
+        f.abort(404)
+    full = f.request.args.get('full', False)
+    
+    #diff = commit.project.git_control.get_file_diff(
+    #    commit.sha1, commit_file.file_path)
+    diff = commit.project.git_control.get_stable_diff(
+        commit.sha1, commit_file.file_path, full_diff=full)
     diff = diff.replace('<','&lt;')
     diff = diff.replace('>','&gt;')
     add_re = re.compile(r'^(\+.*?)$', flags=re.M)
@@ -287,7 +291,7 @@ def commit_file(pid, cid):
     diff = add_re.sub(r'<span class="code-line-added">\1</span>', diff)
     diff = rm_re.sub(r'<span class="code-line-removed">\1</span>', diff)
     return f.render_template('commit_file_diff.html', 
-        commit_file=commit_file, diff=diff, commit=commit)
+        commit_file=commit_file, diff=diff, commit=commit, full=full)
 
 @app.route('/project/<int:pid>/commits')
 @requires_user()
@@ -297,17 +301,24 @@ def commits(pid):
     except:
         f.abort(404)
 
+    page = f.request.args.get('page',1)
+    limit = f.request.args.get('limit',100)
+
     status_filter = f.request.args.get('status_filter','').strip()
     # Peewee queries are extra ugly - not sure why I chose it.
     statuses = [s[0] for s in md.Commit.STATUSES]
     if status_filter:
         statuses = status_filter.split(',')
     
-    print "Statuses: ", statuses
+    #print "Statuses: ", statuses
     commits = md.Commit.select().where(
         (md.Commit.project == project) &
         (md.Commit.status << statuses)
-    )
+    ).order_by(md.Commit.author_date.desc()).paginate(int(page), int(limit))
+    print "Page = %s, limit = %s, count = %s"%(page, limit, commits.count())
+    for c in commits:
+        print "\t", c.id
+    
     return f.render_template('commits.html', project=project, 
         statuses=statuses, commits=commits)
 
